"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fastify_1 = tslib_1.__importDefault(require("fastify"));
const logger_1 = tslib_1.__importDefault(require("./utils/logger"));
const observables_1 = require("./observables");
const botstream_1 = require("./botstream");
const app = fastify_1.default({ logger: logger_1.default, disableRequestLogging: true });
app.get('/api/messages', (req, reply) => {
    // Parse params from the webhook verification request
    const mode = req.query['hub.mode'];
    const token = req.query['hub.verify_token'];
    const challenge = req.query['hub.challenge'];
    // Check if a token and mode were sent
    if (mode && token) {
        // Check the mode and token sent are correct
        if (mode === 'subscribe' && token === botstream_1.globalConfig.facebookVerifyToken) {
            // Respond with 200 OK and challenge token from the request
            reply.send(challenge);
            return;
        }
    }
    reply.status(403);
});
app.post('/api/messages', (req, reply) => {
    // Parse the request body from the POST
    const body = req.body;
    // Return a '404 Not Found' if event is not from a page subscription
    if (body.object !== 'page' || !body.entry) {
        reply.status(404);
        return;
    }
    body.entry.forEach((entry) => {
        const webhookEvent = entry.messaging[0];
        logger_1.default.info(webhookEvent, 'Webhook Event');
        observables_1.event$.next(webhookEvent);
    });
    reply.send();
});
exports.default = app;
